﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace XmlProtocolSampleApp
{
    public partial class SampleEcr : Form
    {
        // List<ProductName, price per item, number of items>
        private List<Tuple<string, int, int>> _products = new List<Tuple<string, int, int>>();

        private BaxiProperties _baxiPropWindow;
        private XmlProtocolController _ctrl;
        private StoreboxReceiptForm storeboxForm;
        private Form pendingUserDialog;

        /// <summary>
        /// Delegates for invoking UI elements from the thread that is receiving the messages from Baxi Agent.
        /// </summary>
        /// <param name="info"></param>
        /// <param name="append"></param>
        delegate void AddDisplayTextDelegate(string info, bool append = true);
        delegate void AddPrintTextDelegate(string info, bool append = true);
        delegate void AddLocalModeDelegate(string info, bool append = true);
        delegate void AddBaxiProperties(Dictionary<string, string> values);
        delegate void AddActivateOperatorConfirmationDelegate(int messageType, string info);
        delegate void AddDisposePendingUserDialogDelegate(Form pendingUserDialog);
        private TerminalConnectionType _terminalConnectionType;
        private string _terminalId;
        private string _ecrId;
        private string _agentIP;
        private string _agentPort;
        private int _agentPortInt;
        private string _username;
        private string _password;


        private bool _isOpenRequest;
        
        public SampleEcr()
        {

            _terminalId = ConfigurationManager.AppSettings["TerminalId"] ?? "70073002";
            _ecrId = ConfigurationManager.AppSettings["EcrId"] ?? "XMLProtocolSample";
            _agentIP = ConfigurationManager.AppSettings["AgentIP"] ?? "127.0.0.1";
            _agentPort = ConfigurationManager.AppSettings["AgentPort"] ?? "6001";
            _username = ConfigurationManager.AppSettings["Username"] ?? "";
            _password = ConfigurationManager.AppSettings["Password"] ?? "";

            Int32.TryParse(_agentPort, out _agentPortInt);
            InitializeComponent();

            TerminalConnectionConfiguration terminalConfig = new TerminalConnectionConfiguration();
            terminalConfig.AgentIP = _agentIP;
            terminalConfig.AgentPort = _agentPortInt;
            terminalConfig.ConnectionType = ConfigurationManager.AppSettings["ConnectionType"] == "cloud" ? TerminalConnectionType.BAXICLOUD : TerminalConnectionType.BAXIAGENT;
            terminalConfig.EcrId = _ecrId;
            terminalConfig.TerminalId = _terminalId;
            terminalConfig.Username = _username;
            terminalConfig.Password = _password;

            _ctrl = new XmlProtocolController(terminalConfig);

            _ctrl.OnDisplayTextReceived += ShowInfoInDisplayTextArea;
            _ctrl.OnMethodRejectedReceived += ShowInfoInDisplayTextArea;
            _ctrl.OnErrorReceived += ShowInfoInDisplayTextArea;
            _ctrl.OnGetProperties += ShowBaxiProperties;
            _ctrl.OnBaxiPropertiesSet += ShowBaxiPropertiesSet;

            _ctrl.OnPrintTextReceived += ShowInfoInPrintTextArea;

            _ctrl.OnLocalModeReceived += ShowInfoInLocalModeArea;
            _ctrl.OnTerminalReadyReceived += ShowInfoInLocalModeArea;
            _ctrl.OnJsonReceived += ShowInfoInLocalModeArea;

            _ctrl.OnPinBypassRequestReceived += ShowOperatorConfirmationDialog;
            pendingUserDialog = null;

            _ctrl.OpenConnection();
            _isOpenRequest = true;
            this.Text += " ITU: " + _terminalId + " ECR: " + _ecrId + " Agent: " + _agentIP + ":" + _agentPort;

            storeboxForm = new StoreboxReceiptForm();
        }

        public class TerminalConnectionConfiguration
        {
            public TerminalConnectionType ConnectionType { get; set; }
            public string TerminalId { get; set; }
            public string EcrId { get; set; }
            public string AgentIP { get; set; }
            public int AgentPort { get; set; }
            public string Username { get; set; }
            public string Password { get; set; }

        }

        public enum TerminalConnectionType
        {
            BAXIAGENT,
            BAXICLOUD
        }



        private void ShowBaxiPropertiesSet(object sender, EventArgs e)
        {
            MessageBox.Show("Baxi properties have been saved!", "Information", MessageBoxButtons.OK);
        }

        private void ShowInfoInLocalModeArea(string info, bool append = true)
        {
            if (InvokeRequired)
                Invoke(new AddLocalModeDelegate(AddLocalModeInfo), new object[] { info, append });
            else
                AddLocalModeInfo(info, append);
        }

        private void AddLocalModeInfo(string info, bool append = true)
        {
            if (_isOpenRequest)
            {
                lblConnectionStatus.Text = info.Contains("Result: 1") ? "Connected" : "Disconnected";
                _isOpenRequest = false;
            }
            if (append)
                richtxtLocalMode.AppendText(info);
            else
                richtxtLocalMode.Text = info;
        }

        private void ShowInfoInPrintTextArea(string info, bool append = true)
        {
            if (InvokeRequired)
                Invoke(new AddPrintTextDelegate(AddPrintTextInfo), new object[] { info, append });
            else
                AddPrintTextInfo(info, append);
            
        }

        private void AddPrintTextInfo(string info, bool append = true)
        {
            if (append)
                richtxtReceiptText.AppendText(info);
            else
                richtxtReceiptText.Text = info;
        }

        private void ShowInfoInDisplayTextArea(string info, bool append = true)
        {
            if (InvokeRequired)
                Invoke(new AddDisplayTextDelegate(AddDisplayTextInfo), new object[] { info, append });
            else
                AddDisplayTextInfo(info, append);
        }

        private void AddDisplayTextInfo(string info, bool append = true)
        {
            if (_isOpenRequest)
            {
                lblConnectionStatus.Text = info.Contains("ALREADY_OPEN") ? "Connected" : "Disconnected";
                _isOpenRequest = false;
            }
            if (append)
                richtxtDisplayText.AppendText(info);
            else
                richtxtDisplayText.Text = info;
        }

        private void ShowOperatorConfirmationDialog(int messageType, string infoText)
        {
            if (InvokeRequired)
            {
                Invoke(new AddActivateOperatorConfirmationDelegate(ActivateOperatorConfirmationDialog), new object[] { messageType, infoText });
            }
            else
            {
                ActivateOperatorConfirmationDialog(messageType, infoText);
            }
        }

                /// <summary>
        /// If BAXI requests POS operator confirmation for some action, we'll show a YES/NO dialog.
        /// </summary>
        /// <param name="messageType">Confirmation message type id (for example: 1 = PIN Bypass)</param>
        /// <param name="append">not used for anything here</param>
        private void ActivateOperatorConfirmationDialog(int messageType, string infoText)
        {            
            ConfirmationDialog dialog = new ConfirmationDialog(messageType, infoText, 10);
            dialog.AddButtonClickListener(new EventHandler(this.OperatorConfirmationDialogButton_Click));

            this.pendingUserDialog = dialog;
            dialog.Show();
        }

        /// <summary>
        /// A click listener to react to operator confirmation dialog yes/no clicks. 
        /// </summary>
        public void OperatorConfirmationDialogButton_Click(object sender, EventArgs e)
        {

            try
            {

                Button clickedButton = (Button)sender;
                ConfirmationDialog buttonOwnerDialog = (ConfirmationDialog)clickedButton.FindForm();

                if (buttonOwnerDialog.AcceptButton == clickedButton) // "yes" was clicked
                {
                    _ctrl.SendConfirmationRequestReply(buttonOwnerDialog.MessageType, true);
                }
                else
                {
                    _ctrl.SendConfirmationRequestReply(buttonOwnerDialog.MessageType, false);
                }

                // no invoke needed here as we're in the same thread
                buttonOwnerDialog.Dispose(); 

            }
            catch (NullReferenceException exception)
            {
                Console.WriteLine("Confirmation dialog had been disposed");
            }

        }

        /// <summary>
        /// Helper methods for checking if there's a pending confirmation dialog when the terminal
        /// has rebooted. The dialog will be killed in that case.
        /// </summary>
        /// <param name="info">Local Mode event info, not used for anything here</param>
        /// <param name="append">not used for anything here</param>
        private void CheckPendingUserDialog(string info, bool append)
        {
            // terminal rebooted while awaiting user input?
            if (pendingUserDialog != null)
            {
                if (InvokeRequired)
                {
                    Invoke(new AddDisposePendingUserDialogDelegate(DisposePendingUserDialog), new object[] { pendingUserDialog });
                    this.pendingUserDialog = null;
                }
            }
        }

        private void DisposePendingUserDialog(Form pendingUserDialog)
        {
            pendingUserDialog.Dispose();
        }



        private void ShowBaxiProperties(Dictionary<string, string> values)
        {
            if (InvokeRequired)
                Invoke(new AddBaxiProperties(OpenBaxiPropertiesWindow), new object[] { values });
            else
                OpenBaxiPropertiesWindow(values);
        }

        private void OpenBaxiPropertiesWindow(Dictionary<string, string> values)
        {
            _baxiPropWindow = new BaxiProperties(_ctrl, values);
            _baxiPropWindow.Show();
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            _products = new List<Tuple<string, int, int>>();
            txtTotalAmount.Text = "";
            richtxtItems.Text = "";
            ClearRichTextBoxes();
        }

        private void ClearRichTextBoxes()
        {
            richtxtDisplayText.Text = "";
            richtxtLocalMode.Text = "";
            richtxtReceiptText.Text = "";
        }

        private void btnApple_Click(object sender, EventArgs e)
        {
            AddProduct("Apple", 100);
            BuildProductList();
        }

        private void btnOrange_Click(object sender, EventArgs e)
        {
            AddProduct("Orange", 50);
            BuildProductList();
        }

        private void btnBeer_Click(object sender, EventArgs e)
        {
            AddProduct("Beer", 150);
            BuildProductList();
        }

        private void btnJuice_Click(object sender, EventArgs e)
        {
            AddProduct("Juice", 55);
            BuildProductList();
        }

        private void richtxtItems_TextChanged(object sender, EventArgs e)
        {
            var amount = 0;
            foreach (var item in _products)
            {
                amount = amount + (item.Item2 * item.Item3);
            }
            if (amount == 0)
            {
                txtTotalAmount.Text = "";
            }
            else
            {
                txtTotalAmount.Text = amount.ToString();
            }
        }

        private void AddProduct(string name, int price)
        {
            var item = _products.FirstOrDefault(x => x.Item1 == name);
            if (item != null)
            {
                var count = item.Item3 + 1;
                _products.Remove(item);
                item = new Tuple<string, int, int>(name, price, count);
                _products.Add(item);
            }

            else
            {
                item = new Tuple<string, int, int>(name, price, 1);
                _products.Add(item);
            }
        }

        private void BuildProductList()
        {
            richtxtItems.Text = string.Empty;
            foreach (var item in _products)
            {
                richtxtItems.Text = richtxtItems.Text +
                    string.Format("{0} \n{1}  X  {2} = {3}\n", item.Item1, (item.Item2 * 0.01).ToString("F"), item.Item3.ToString("F"), (item.Item3 * item.Item2 * 0.01).ToString("F"));
            }
        }

        private void btnCardPayment_Click(object sender, EventArgs e)
        {
            if (!string.IsNullOrEmpty(txtTotalAmount.Text))
            {
                ClearRichTextBoxes();
                _ctrl.CardPayment(txtTotalAmount.Text, "48");
            }
            else
            {
                MessageBox.Show("Please select at least one product!");
            }
        }

        private void btnCardInfo_Click(object sender, EventArgs e)
        {
            ClearRichTextBoxes();
            _ctrl.GetCardInfo();
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            ClearRichTextBoxes();
            _ctrl.AdminCancel();
        }

        private void btnEndOfDay_Click(object sender, EventArgs e)
        {
            ClearRichTextBoxes();
            _ctrl.EndOfDay();
        }

        private void btnLastFinancialReceipt_Click(object sender, EventArgs e)
        {
            ClearRichTextBoxes();
            _ctrl.LastFinancialReceipt();
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            _ctrl.CloseConnection();
            this.Close();
        }

        private void btnBonusTransaction_Click(object sender, EventArgs e)
        {
            if (!string.IsNullOrEmpty(txtTotalAmount.Text))
            {
                ClearRichTextBoxes();
                _ctrl.CardPayment(txtTotalAmount.Text, "62");
            }
            else
            {
                MessageBox.Show("Please select at least one product!");
            }

        }

        private void btnTransferCardData_Click(object sender, EventArgs e)
        {
            if (!string.IsNullOrEmpty(txtCardData.Text))
            {
                ClearRichTextBoxes();
                _ctrl.TransferCardData(txtCardData.Text);
            }
            else
            {
                MessageBox.Show("Please input card data");
            }

        }

        private void toolStripMenuItemProperties_Click(object sender, EventArgs e)
        {
            _ctrl.GetRWProperties();

        }

        private void btnStorebox_Click(object sender, EventArgs e)
        {
            if (!string.IsNullOrEmpty(txtTotalAmount.Text))
            {
                var json = _ctrl.BuildStoreboxReceipt(_products);
                storeboxForm.SetJson(json);
                if (storeboxForm.ShowDialog(this) == DialogResult.OK)
                {
                    _ctrl.SendStoreBoxReceipt(storeboxForm.GetJson());
                }
            }
            else
            {
                MessageBox.Show("Please select at least one product!");
            }
        }

    }
}
